# -*- coding: utf-8 -*-

from django.conf import settings as _settings

from meringue.conf import default_settings


class Settings:

    def __init__(self, default=default_settings):

        for setting in default.__all__:
            if not setting.isupper():
                continue

            default_value = getattr(default, setting)
            setting_value = getattr(_settings, f'MERINGUE_{setting}', default_value)
            setattr(self, setting, setting_value)


settings = Settings()



# -*- coding: utf-8 -*-

import logging  # noqa

from django.core.paginator import Paginator as DjPaginator


class Paginator(DjPaginator):
    """
    What?
    """

    def __init__(self, object_list, per_page, orphans=0,
                 allow_empty_first_page=True, reverse=False):
        super().__init__(object_list, per_page, orphans,
                         allow_empty_first_page)
        self.reverse = bool(reverse)

    def page(self, number):
        """Return a Page object for the given 1-based page number."""
        number = self.validate_number(number)

        bottom = (number - 1) * self.per_page
        top = bottom + self.per_page
        if top + self.orphans >= self.count:
            top = self.count

        if self.reverse:
            bottom, top = self.count - top, self.count - bottom

        return self._get_page(self.object_list[bottom:top], number, self)



# -*- coding: utf-8 -*-
"""
See PEP 386 (https://peps.python.org/pep-0440/)

Release logic:
 1. Remove ".devX" from __version__ (below)
 2. git add treebeard/__init__.py
 3. git commit -m 'Bump to <version>'
 4. git tag <version>
 5. git push
 6. assure that all tests pass on https://travis-ci.org/django-treebeard/django-treebeard/builds/
 7. git push --tags
 8. pip install --upgrade pip wheel twine
 9. python setup.py clean --all
 9. python setup.py sdist bdist_wheel
10. twine upload dist/*
11. bump the version, append ".dev0" to __version__
12. git add treebeard/__init__.py
13. git commit -m 'Start with <version>'
14. git push
"""

__version__ = "0.4.0.post1"



# -*- coding: utf-8 -*-

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class Config(AppConfig):
    name = 'meringue.core'
    verbose_name = _('Зефирка')
    label = 'meringue'



# -*- coding: utf-8 -*-

import logging  # noqa
import re


# cm
def set_crop_method(method, opt):
    return {'crop_method': method.split(' ')}


# rm
def set_resize_method(method, opt):
    return {'resize_method': method}


# s
def set_size(raw, opt):
    size_re = re.compile(r'(\d+)x(\d+)')
    size = [float(i) for i in size_re.search(raw).groups()]

    # расчитываем относительные размеры если один из размеров не указан
    size[0] = size[0] if size[0] else size[1] * opt['current_size'][0] / \
        opt['current_size'][1]
    size[1] = size[1] if size[1] else size[0] * opt['current_size'][1] / \
        opt['current_size'][0]

    return {'new_size': size}


# maxw
def set_max_width(width, opt):
    return {'max_width': float(width)}


# maxh
def set_max_height(height, opt):
    return {'max_height': float(height)}


# q
def set_quality(quality, opt):
    return {'quality': int(quality)}


# c
def set_bg_color(color, opt):
    tmp = tuple()
    for i in color.split(' '):
        tmp += (int(i), )
    return {'bg_color': tmp}


property_list = {
    'cm': set_crop_method,
    'rm': set_resize_method,
    's': set_size,
    'maxw': set_max_width,
    'maxh': set_max_height,
    'q': set_quality,
    'c': set_bg_color,
}



# -*- coding: utf-8 -*-

from PIL import Image
import logging
import os.path
import re
import six
try:
    from hashlib import md5
except ImportError:
    from md5 import md5

from django.conf import settings as django_settings

from .methods import method_list
from .properties import property_list
from meringue.conf import settings


logger = logging.getLogger('meringue')


property_list.update(settings.THUMBNAIL_PROPERTIES)
method_list.update(settings.THUMBNAIL_METHODS)


class Thumbnail(object):

    options = {
        'crop_method': settings.THUMBNAIL_CROP_METHOD,
        'resize_method': settings.THUMBNAIL_RESIZE_METHOD,
        'quality': settings.THUMBNAIL_QUALITY,
        'color': settings.THUMBNAIL_COLOR,
        'bg_color': settings.THUMBNAIL_BG_COLOR,
        'max_width': None,
        'max_height': None,
    }

    store_dir = settings.THUMBNAIL_DIR
    store_url = settings.THUMBNAIL_URL

    def __init__(self, filename, proc='extra@1'):
        if os.path.isabs(filename):
            self.filename = filename
        else:
            self.filename = os.path.join(django_settings.MEDIA_ROOT, filename)

        if not os.path.isfile(os.path.realpath(self.filename)):
            raise Exception('File \'%s\' does not exist.' % filename)
        self.proc = proc
        if not self.is_valid_thumbnail():
            self.make_thumbnail()
        else:
            self.image = Image.open(self.thumbnail_filename)

    def is_valid_thumbnail(self):
        '''
            проверяет существование и актуальность превью.
        '''
        if not settings.THUMBNAIL_DEBUG and \
           os.path.isfile(self.thumbnail_filename):
            return os.path.getmtime(self.filename) < os.path.getmtime(
                self.thumbnail_filename
            )
        return False

    def get_thumbnail_filename(self):
        if getattr(self, '_thumbnail_filename', None) is None:
            filename = '%s.png' % (self.get_hash(), )
            if not os.path.exists(self.store_dir):
                os.mkdir(self.store_dir)
            self._thumbnail_filename = os.path.join(self.store_dir, filename)
        return self._thumbnail_filename
    thumbnail_filename = property(get_thumbnail_filename)

    def get_hash(self):
        '''
            Указывать конечный размер?? зачем?
        '''
        hash = [i for i in self.proc]  # self.proc
        hash.append(os.path.basename(self.filename))
        return md5(str(hash).encode()).hexdigest()

    def make_thumbnail(self):
        # try:
        self.image = Image.open(self.filename)
        self.image = self.image.convert('RGBA')
        # except IOError, detail:
        #     raise Exception(detail)

        self.options['current_size'] = [float(i) for i in self.image.size]
        self.options['new_size'] = [float(i) for i in self.image.size]

        logger.info("make thumbnail with tasks: %s" % ", ".join(self.proc))
        for proc in self.proc:
            if ':' in proc:
                prop, args = proc.split(':')
                self.options.update(property_list[prop](args, self.options))

            elif proc in method_list:
                self.image = method_list[proc](self.image, self.options)

        self.image.save(self.thumbnail_filename, 'PNG',
                        quality=self.options['quality'], optimize=1)

    # # готовые решения
    # def _thumbnail(self):
    #     pass

    def get_thumbnail_url(self):
        if getattr(self, '_thumbnail_url', None) is None:
            self._thumbnail_url = "%s%s" % (
                self.store_url,
                os.path.basename(self.thumbnail_filename)
            )
        return self._thumbnail_url
    thumbnail_url = property(get_thumbnail_url)


def _dummyimage(task_list):
    if django_settings.DEBUG:
        task_list.reverse()
        final_size = re.compile(r's:(\d+)x(\d+)')
        for task in task_list:
            try:
                size = final_size.search(task).groups()
                break
            except AttributeError:
                pass
        # TODO: BUG: size could be undefined
        return u'http://dummyimage.com/%sx%s/9e9e9e/424242.png' % size
    return '%simages/noise.png' % django_settings.STATIC_URL
    # return '%simages/none.gif' % django_settings.STATIC_URL


def get_thumbnail(filename, task_list):
    if isinstance(task_list, six.string_types):
        task_list = task_list.split(',')
    url = ''
    if os.path.isfile(filename):
        thumb = Thumbnail(filename, task_list)
        url = thumb.thumbnail_url
        # try:
        # except Exception, er:
        #     # TODO 1: log error message cuz noone wishes to debug till here
        #     # TODO 2: Если в размерах ноль при ошибке нет превьюшки
        #     logger.error(Exception)
        #     logger.error(er)
        #     url = _dummyimage(task_list)
    else:
        logger.error(u'File \'%s\' not found' % filename)
        # находить последний размер и цвет фона
        url = _dummyimage(task_list)
        # return ''
    return url



# -*- coding: utf-8 -*-

from django import template
from django.utils import timezone
from django.utils.safestring import mark_safe

from meringue.conf import settings


register = template.Library()


@register.simple_tag
def cop_year():
    """
    return range of years for copyright in one of the following formats:
        YYYY-YYYY
        YYYY

    set start year for parametr MERINGUE_START_YEAR in setting.py
    """

    year = timezone.now().year
    return year == settings.START_YEAR and year or \
        mark_safe('%d&mdash;%d' % (settings.START_YEAR, year))


@register.simple_tag
def date_range(date_start, date_end):
    """
    return range of date in one of the following formats:
        DD MM YYYY - DD MM YYYY
        DD MM - DD MM YYYY
        DD - DD MM YYYY
    """

    if date_start.year != date_end.year:
        result = '&nbsp;&mdash; '.join([date_start.strftime('%d %B %Y'),
                                        date_end.strftime('%d %B %Y')])
    elif date_start.month != date_end.month:
        result = '&nbsp;&mdash; '.join([date_start.strftime('%d %B'),
                                        date_end.strftime('%d %B %Y')])
    elif date_start.day != date_end.day:
        result = '&mdash;'.join([date_start.strftime('<nobr>%d'),
                                 date_end.strftime('%d %B</nobr> %Y')])
    else:
        result = date_end.strftime('%d %B %Y')

    return mark_safe(result)



# -*- coding: utf-8 -*-

from django.db.models.manager import Manager

from meringue.core.query import PublishQuerySet


class PublishManager(Manager):
    use_for_related_fields = True

    def get_queryset(self):
        return PublishQuerySet(self.model, using=self._db)

    def published(self, *args, **kwargs):
        return self.get_queryset().published(*args, **kwargs)

    def unpublished(self, *args, **kwargs):
        return self.get_queryset().unpublished(*args, **kwargs)



# -*- coding: utf-8 -*-

import logging  # noqa
import math

from PIL import Image


def get_size(options, max_sizes=True):
    new_size = options['new_size']
    if max_sizes:
        if options['max_width'] and new_size[0] > options['max_width']:
            new_size[0] = options['max_width']
        if options['max_height'] and new_size[1] > options['max_height']:
            new_size[1] = options['max_height']
    return new_size


def crop(image, options):
    new_size = get_size(options)
    new_size = [int(new_size[0]), int(new_size[1])]

    current_size = options['current_size']
    thumb = Image.new(mode='RGBA', size=new_size, color=options['bg_color'])

    # позиционируем новое изображение
    if options['crop_method'][0] == 'left':
        x = 0
    elif options['crop_method'][0] == 'right':
        x = new_size[0] - current_size[0]
    else:
        x = (new_size[0] - current_size[0]) / 2

    if options['crop_method'][1] == 'left':
        y = 0
    elif options['crop_method'][1] == 'right':
        y = new_size[1] - current_size[1]
    else:
        y = (new_size[1] - current_size[1]) / 2

    thumb.paste(image, (int(x), int(y)))
    image = thumb
    options['current_size'] = new_size

    return image


def resize(image, options):
    new_size = get_size(options)
    current_size = options['current_size']

    if options['resize_method'] in ['cover', 'contain']:

        # заполнить всё новое изображение
        if options['resize_method'] == 'cover':
            res = max(new_size[0] / current_size[0],
                      new_size[1] / current_size[1])

        # вписать изображение полностью
        elif options['resize_method'] == 'contain':
            res = min(new_size[0] / current_size[0],
                      new_size[1] / current_size[1])

        new_size = [math.floor(current_size[0] * res),
                    math.floor(current_size[1] * res)]

        # при сжатии никак не меняем конечный размер

    new_size = [int(i) for i in new_size]
    image = image.resize(new_size, resample=Image.ANTIALIAS)
    options['current_size'] = new_size

    return image


method_list = {
    'crop': crop,
    'resize': resize,
}



# -*- coding: utf-8 -*-

from django.core.files import uploadhandler

from meringue.conf import settings


class MemoryFileUploadHandler(uploadhandler.MemoryFileUploadHandler):
    def file_complete(self, file_size):
        result = super(MemoryFileUploadHandler, self).file_complete(file_size)
        if result:
            result.name = settings.UPLOAD_RENAME_HANDLER(result)
            result.file.seek(0)
        return result


class TemporaryFileUploadHandler(uploadhandler.TemporaryFileUploadHandler):
    def file_complete(self, file_size):
        result = super(TemporaryFileUploadHandler, self).\
            file_complete(file_size)
        if result:
            result.name = settings.UPLOAD_RENAME_HANDLER(result)
            result.file.seek(0)
        return result



# -*- coding: utf-8 -*-

from importlib import import_module

from django.apps import apps
from django.conf import settings

from modeltranslation.translator import translator, TranslationOptions
try:
    from polymorphic.utils import get_base_polymorphic_model
except ImportError:
    get_base_polymorphic_model = None


for app in settings.INSTALLED_APPS:
    try:
        models = import_module('%s.models' % app)
    except ImportError:
        bits = app.split('.')
        app = import_module(bits[0])

        for bit in bits[1:]:
            app = getattr(app, bit)

        if hasattr(app, 'name'):
            try:
                models = import_module('%s.models' % app.name)
            except ImportError:
                continue
        else:
            continue
    except AttributeError:
        continue

    for model in apps.get_models():
        fields = getattr(model._meta, 'translate_fields', list())
        force = False

        if not fields and get_base_polymorphic_model and get_base_polymorphic_model(model):
            force = True

        if (fields or force) and not model in translator.get_registered_models():
            translator.register(
                model,
                type(
                    str(model.__name__ + 'Translation'),
                    (TranslationOptions, ),
                    {'fields': fields}
                )
            )



# -*- coding: utf-8 -*-

import django.db.models.options as options


options.DEFAULT_NAMES = options.DEFAULT_NAMES + (
    'translate_fields',
    # 'host_name',
    # 'view',
    # 'reverse_args',
)



# -*- coding: utf-8 -*-

from django.db.models.query import QuerySet


class PublishQuerySet(QuerySet):
    use_for_related_fields = True

    def published(self, *args, **kwargs):
        kwargs.update({
            'is_published': True
        })
        return self.filter(*args, **kwargs)

    def unpublished(self, *args, **kwargs):
        kwargs.update({
            'is_published': False
        })
        return self.filter(*args, **kwargs)



# -*- coding: utf-8 -*-

import logging  # noqa

# from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

# if 'django_hosts' in settings.INSTALLED_APPS:
#     from django_hosts.resolvers import reverse
# else:
#     from django.urls import reverse

from .managers import PublishManager
# from meringue import configuration


logger = logging.getLogger('meringue')


##########
# mixins #
##########

class CMTimeMixin(models.Model):
    ctime = models.DateTimeField(auto_now_add=True)
    mtime = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class SortingMixin(models.Model):
    sorting = models.SmallIntegerField(verbose_name=_('Порядок'),
                                       help_text=_('Порядок сортировки'),
                                       default=0, )
    class Meta:
        ordering = ['sorting', ]
        abstract = True


# class GetAbsoluteUrlMixin(object):

#     '''
#         Примесь автоматизирующая получение абсолютного адреса
#         Для получения адреса использует стандартную функцию reverse (в
#     случае если установлен django_hosts то reverse_full)

#         При реверсе предполагается следующая схема:

#             * host_name - название верхнего модуля в нижнем регистре
#         (используется в случае установенного django_hosts)

#             * view - '<namespace>:<url_name>':
#                 - namespace - название приложения приведённое к нижнему
#             регистру и точки заменённые на нижнее подчёркивание
#                 - url_name - название класса приведённое к нижнему регистру
#             с приставкой '-detail'

#             * args - по умолчанию pk модели

#         При определении класса модели можно предустановить параметры:
#             * host_name - название целевого хоста (string)
#             * view - целевой url (string)
#             * reverse_args - список атрибутов модели (list)
#     '''

#     def _host_name(self):
#         if not hasattr(self._meta, 'host_name'):
#             module_name = self.__module__.split('.')[0]
#             module = __import__(module_name)
#             self._meta.host_name = getattr(module, 'host_name',
#                                            module_name.lower())
#         return self._meta.host_name

#     def _view(self):
#         if not hasattr(self._meta, 'view'):
#             namespace = self.__module__[:-7].lower().replace('.', '_')

#             cls = self._meta.model_name
#             view = '%s-detail' % cls

#             self._meta.view = '%s:%s' % (namespace, view)
#         return self._meta.view

#     def _reverse_args(self):
#         reverse_args = getattr(self._meta, 'reverse_args', ['pk'])
#         return [getattr(self, key) for key in reverse_args]

#     def get_absolute_url(self):
#         reverse_args = {
#             'viewname': self._view(),
#             'args': self._reverse_args(),
#         }
#         if 'django_hosts' in settings.INSTALLED_APPS:
#             reverse_args.update({
#                 'host': self._host_name(),
#                 'port': configuration.PORT,
#             })
#         return reverse(**reverse_args)


###################
# Abstract models #
###################

class PublishedBase(models.Model):
    """
        publish abstract model
        managr has method published() like a filter()
    """

    is_published = models.BooleanField(
        verbose_name=_('публикация'),
        help_text=_('Отображать/Скрыть'),
        default=True,
        db_index=True,
    )

    objects = PublishManager()

    class Meta:
        abstract = True



# -*- coding:utf-8 -*-

import os
import uuid

from django.conf import settings
from django.utils import timezone


__all__ = ['THUMBNAIL_DEBUG', 'THUMBNAIL_PROPERTIES', 'THUMBNAIL_METHODS', 'THUMBNAIL_CROP_METHOD',
           'THUMBNAIL_RESIZE_METHOD', 'THUMBNAIL_QUALITY', 'THUMBNAIL_COLOR', 'THUMBNAIL_BG_COLOR',
           'THUMBNAIL_DIR', 'THUMBNAIL_URL', 'START_YEAR',]


########
# CORE #
########


# upload handlers #

def UPLOAD_RENAME_HANDLER(result):
    return u'{0}{1}'.format(uuid.uuid4(), os.path.splitext(result.name)[1])


# thumbnail #

THUMBNAIL_DEBUG = False
THUMBNAIL_PROPERTIES = {}
THUMBNAIL_METHODS = {}
THUMBNAIL_CROP_METHOD = ['center', 'center']
THUMBNAIL_RESIZE_METHOD = 'cover'
THUMBNAIL_QUALITY = 100
THUMBNAIL_COLOR = (255, 255, 255, 0)
THUMBNAIL_BG_COLOR = (200, 200, 200, 0)
THUMBNAIL_DIR = os.path.join(settings.MEDIA_ROOT, 'temp')
THUMBNAIL_URL = settings.MEDIA_URL + 'temp/'


# copyright year #

START_YEAR = timezone.now().year



# -*- coding:utf-8 -*-


def unify_email(origin_email):
    """
        unify email address
        remove tags
    """
    if not origin_email or not '@' in origin_email[1:-3]:
        return None
    user, domain = origin_email.strip().rsplit('@', 1)
    user = user.split('+')[0]
    email = '@'.join([user, domain])
    email = email.lower()
    return email


def unify_phone(origin_phone):
    """
        default country code use if cant parse phone number
    """

    import phonenumbers as pn

    try:
        phone = pn.parse(origin_phone)
    except pn.phonenumberutil.NumberParseException:
        phone = pn.parse(origin_phone, default_country)

    result = pn.format_number(phone, pn.PhoneNumberFormat.INTERNATIONAL)
    return result
